local s = SRB2Smash

-- Badnik Trophy
-- Throwable capsule that spawns one of 4 badniks
-- Crawla, Pop-Up Turret, Jetty Syn Gunner, Jetty Syn Bomber

freeslot("MT_SMASH_LIFE_LOCKON", "S_SMASHLOCKON", "MT_BADNIKTROPHY", "S_CAPSULE", "S_CAPSULE_DESTROY")

mobjinfo[MT_SMASH_LIFE_LOCKON] = {
    spawnstate = S_SMASHLOCKON,
    radius = 1*FRACUNIT,
    height = 1*FRACUNIT,
	flags = MF_NOCLIP|MF_NOGRAVITY|MF_RUNSPAWNFUNC
}

states[S_SMASHLOCKON] = {SPR_PLAY, FF_FULLBRIGHT|SPR2_LIFE, -1, nil, 0, 0, S_SMASHLOCKON}

local function A_SpawnRandBadnik(actor)
	local badniks = {MT_BLUECRAWLA, MT_POPUPTURRET, MT_JETTGUNNER, MT_JETTBOMBER, MT_UNIDUS}

	local mo = P_SpawnMobj(actor.x, actor.y, actor.z, badniks[P_RandomRange(1, #badniks)])
	mo.scale = 1
	mo.destscale = FRACUNIT

	mo.user = actor.target or actor.user
	mo.fuse = 700
	
	mo.flags = $ & ~MF_SHOOTABLE
	
	for i = 1, 20
		local spark = P_SpawnMobj(actor.x, actor.y, actor.z, MT_SUPERSPARK)
		spark.momx = P_RandomRange(-20, 20)*FRACUNIT
		spark.momy = P_RandomRange(-20, 20)*FRACUNIT
		spark.momz = P_RandomRange(0, 20)*FRACUNIT
	end
end

mobjinfo[MT_BADNIKTROPHY] = {
    spawnstate = S_CAPSULE,
	deathstate = S_CAPSULE_DESTROY,
    radius = 32*FRACUNIT,
    height = 32*FRACUNIT,
	speed = 7*FRACUNIT,
	flags = MF_MISSILE--|MF_GRENADEBOUNCE
}

states[S_CAPSULE] = {SPR_IDYA, A, -1, nil, 0, 0, S_CAPSULE_DESTROY}
states[S_CAPSULE_DESTROY] = {SPR_IDYA, A, 32, A_SpawnRandBadnik, 7, 2, S_NULL}

s.mobjDefs[MT_BADNIKTROPHY] = {
	hitsound = sfx_punch2,
	hitstun = 10,
	hitlag = 5,
	hitcool = 4,
	x = 8*FRACUNIT,
	y = 5*FRACUNIT,
	mod = true,
	dmg = 10,
}

-- Crawla
s.mobjDefs[MT_BLUECRAWLA] = {
	hitsound = sfx_punch3,
	hitstun = 14,
	hitlag = 7,
	hitcool = 4,
	x = 10*FRACUNIT,
	y = 3*FRACUNIT,
	mod = true,
	dmg = 15,
}

-- Pop-Up Turret
s.mobjDefs[MT_POPUPTURRET] = {
	hitsound = sfx_kick1,
	hitstun = 3,
	hitlag = 3,
	hitcool = 3,
	x = 2*FRACUNIT,
	y = 7*FRACUNIT,
	mod = false,
	dmg = 5,
}

s.mobjDefs[MT_JETTBULLET] = {
	hitsound = sfx_kick3,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 15*FRACUNIT,
	y = 7*FRACUNIT,
	mod = true,
	dmg = 10
}

-- Jetty Syn Gunner
s.mobjDefs[MT_JETTGUNNER] = {
	hitsound = sfx_kick1,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 10*FRACUNIT,
	y = FRACUNIT,
	mod = true,
	dmg = 10,
}

-- Jetty Syn Bomber
s.mobjDefs[MT_JETTBOMBER] = {
	hitsound = sfx_kick1,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 1,
	y = -10*FRACUNIT,
	mod = false,
	dmg = 10,
}

s.mobjDefs[MT_MINE] = {
	hitsound = sfx_fire3,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 1,
	y = -20*FRACUNIT,
	mod = false,
	dmg = 15,
	effect = "fire"
}

-- Unidus
s.mobjDefs[MT_UNIDUS] = {
	hitsound = sfx_kick1,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 5*FRACUNIT,
	y = 4*FRACUNIT,
	mod = false,
	dmg = 4,
}

s.mobjDefs[MT_UNIBALL] = {
	hitsound = sfx_sword2,
	hitstun = 5,
	hitlag = 5,
	hitcool = 10,
	x = 10*FRACUNIT,
	y = 1*FRACUNIT,
	mod = true,
	dmg = 10,
}

addHook("MobjThinker", function(mo)
	if not P_IsObjectOnGround(mo)
		mo.rollangle = $+ANG2
		P_SpawnGhostMobj(mo)
	end
end, MT_BADNIKTROPHY)

addHook("MobjThinker", function(mo)
	if mo.target and mo.target.valid
		P_TeleportMove(mo, mo.target.x, mo.target.y, mo.target.z + 50*FRACUNIT)
		
		if mo.target.user and mo.target.user.valid and mo.target.user.skin
			mo.skin = mo.target.user.skin
			mo.color = mo.target.user.color
			mo.sprite = SPR_PLAY
			mo.sprite2 = SPR2_LIFE
		end
	else
		P_RemoveMobj(mo)
		return
	end
end, MT_SMASH_LIFE_LOCKON)

for i = MT_BLUECRAWLA, MT_DRAGONBOMBER
	addHook("MobjThinker", function(mo)
		if not mo.user or not mo.user.valid then return end

		if not mo.lockon or not mo.lockon.valid
			mo.lockon = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SMASH_LIFE_LOCKON)
			mo.lockon.target = mo
		end

		if not mo.target or mo.target == mo.user
			mo.target = s.closestMobj(mo.user, mo.user)
		end
	end, i)
end