local s = SRB2Smash

s.hitboxThinker = function(p)
	if not p.hitbox then return end

	for i = 1, #p.hitbox
		if p.hitbox[i] and p.hitbox[i].valid
			P_TeleportMove(p.hitbox[i], p.mo.x+(p.hitbox[i].hitbox.xstick or 0), p.mo.y+(p.hitbox[i].hitbox.ystick or 0), p.mo.z+(p.hitbox[i].hitbox.zstick or 0))
		else
			table.remove(p.hitbox, i)
		end
	end
end

s.drawHitBox = function(p, x, y, z, r, high, dmg, prio, kbx, kby, kbmod, hitsound, hiteff, status)
	local h = P_SpawnMobjFromMobj(p.mo, x, y, z, MT_HITBOXSMASH)
	h.angle = p.mo.angle
	h.user = p
	h.usermo = p.mo
	h.radius = r or 20*FRACUNIT
	h.height = high or 20*FRACUNIT
	h.fuse = TICRATE
	
	h.hitcool = 10
	h.atkwait = 6

	-- ring damage buff
	local totDmg = dmg
	if p.mo.smashRings and s.doRingBuff(p.mo)
		if dmg > 5 and dmg <= 15
			totDmg = $+(p.mo.smashRings/10)
		elseif dmg > 15
			totDmg = $+(p.mo.smashRings/5)
		end
	end

	h.hitbox = {
		user = p,
		xstick = x and FixedMul(x, p.mo.scale) or 0,
		ystick = y and FixedMul(y, p.mo.scale) or 0,
		zstick = z and FixedMul(z, p.mo.scale) or 0,
		xfling = kbx and FixedMul(kbx, p.mo.scale) or nil,
		yfling = kby and FixedMul(kby, p.mo.scale) or nil, 
		damage = totDmg,
		priority = tonumber(prio),
		sound = hitsound or sfx_pstop,
		modify = kbmod,
		effect = hiteff,
		status = status or nil
	}
	
	h.status = status or nil
	h.priority = tonumber(prio)

	if not p.hitbox then p.hitbox = {} end
	if p.powers[pw_super] then h.hitbox.damage = $*3 end

	h.scale = p.mo.scale

	p.hitbox[#p.hitbox+1] = h
	return h
end

s.drawGhostHitBox = function(x, y, z, r, high, dmg, prio, kbx, kby, kbmod, hitsound, hiteff)
	local h = P_SpawnMobj(x, y, z, MT_HITBOXSMASH)
	h.radius = r or 20*FRACUNIT
	h.height = high or 20*FRACUNIT
	h.fuse = 5
	
	h.hitcool = 10
	h.atkwait = 6

	h.hitbox = {
		xstick = x,
		ystick = y,
		zstick = z,
		damage = dmg,
		priority = prio,
		xfling = kbx,
		yfling = kby,
		sound = hitsound or sfx_pstop,
		modify = kbmod,
	}

	return h
end

s.hitboxCollide = function(h, p)
	if p and p.mo and p.mo.atkwait then return false end
	if s.noHit(p.mo) then return false end
	
	if s.isTeam() and h.user
		if s.sameTeam(p, h.user) then return false end
	end
	
	if h.usermo and h.usermo.valid and h.usermo.player and s.isAdventure() then return false end

	if p.mo.parry and p.mo.parry > 30 and h.usermo and h.usermo.valid and h.usermo.attacktype ~= "finalsmash"
		p.mo.hitcool = 35
		s.parryStun(h.usermo, p.mo, h.hitbox and h.hitbox.priority or 1)
		return
	end

	if not h or not h.valid then return end
	if not h.hitbox then return end

	if not h.usermo then return s.ghostHitbox(p, h) end

	if h.hitbox and p.mo and p.mo.valid
		if p.mo.hitcool then return false end

		local hit = h.hitbox
		if h.user == p then return false end
		if p.mo.priority <= hit.priority
			if h.usermo and h.usermo.valid
				// Clanking
				if h.usermo.attacktype and p.mo.attacktype
					if (h.usermo.attacktype == p.mo.attacktype) and p.hitbox[1] and h.priority == p.hitbox[1].priority
						S_StartSound(h, sfx_pstop)
						
						for i = 1, 10
							local spark = P_SpawnMobj(h.x, h.y, h.z, MT_SUPERSPARK)
							spark.momx = P_RandomRange(-10, 10)*FRACUNIT
							spark.momy = P_RandomRange(-10, 10)*FRACUNIT
							spark.momz = P_RandomRange(-10, 10)*FRACUNIT
						end

						local hToP = R_PointToAngle2(h.usermo.x, h.usermo.y, p.mo.x, p.mo.y)
						local pToH = R_PointToAngle2(p.mo.x, p.mo.y, h.usermo.x, h.usermo.y)

						P_SetObjectMomZ(p.mo, 4*FRACUNIT)
						P_SetObjectMomZ(h.usermo, 4*FRACUNIT)
						P_InstaThrust(p.mo, hToP, 6*FRACUNIT)
						P_InstaThrust(h.usermo, pToH, 6*FRACUNIT)
						
						p.mo.state = S_PLAY_FALL
						h.usermo.state = S_PLAY_FALL
						
						// End attacks early
						p.mo.attacktype = "none"
						h.usermo.attacktype = "none"
						
						p.mo.hitstun = 5
						p.mo.hitcool = 15
						h.usermo.hitstun = 5
						h.usermo.hitcool = 15

						for i = 1, #p.hitbox
							if p.hitbox[i] and p.hitbox[i].valid then P_RemoveMobj(p.hitbox[i]) end
						end
						p.hitbox = {}
						
						if not h.usermo.player then P_RemoveMobj(h) return false end
						local p2 = h.usermo.player
						
						for i = 1, #p2.hitbox
							if p2.hitbox[i] and p2.hitbox[i].valid then P_RemoveMobj(p2.hitbox[i]) end
						end
						p2.hitbox = {}

						// Stop checking
						return false
					end
				end

				local dmg = hit.damage
				
				if h.usermo.staleMove[2] >= 3 then dmg = $/2 end

				if hit.status == EFF_BURNABSORB and p.mo.statusEffect and p.mo.statusEffect[1] == EFF_BURN
					if h.hitlag then h.hitlag = ($*3)/2 end

					dmg = ($*3)/2
--					p.mo.burnAbsorbed = true
					
					local thok = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_THOK)
					thok.destscale = 3*FRACUNIT
					thok.tics = 20
					thok.fuse = 50
					thok.color = SKINCOLOR_RED

					for i = 1, 30
						local fire = P_SpawnMobj(p.mo.x + P_RandomRange(-20, 20)*FRACUNIT, p.mo.y + P_RandomRange(-20, 20)*FRACUNIT, p.mo.z + P_RandomRange(-20, 20)*FRACUNIT, MT_THOK)
						fire.sprite = SPR_FPRT
						fire.frame = A
						fire.tics = 35
						fire.fuse = 35
						fire.scale = P_RandomRange(3, 5)*P_RandomRange(FRACUNIT/2, FRACUNIT)

						fire.momx = P_RandomRange(-10, 10)*FRACUNIT
						fire.momz = P_RandomRange(-10, 10)*FRACUNIT
					end

					p.mo.statusEffect = nil
				end

				s.addDamage(p.mo, h.usermo, dmg)
			end

			if hit.priority == 2
				P_StartQuake(3*FRACUNIT, 10, {p.mo.x, p.mo.y, p.mo.z}, 200*FRACUNIT)
			elseif hit.priority >= 3
				P_StartQuake(7*FRACUNIT, 10, {p.mo.x, p.mo.y, p.mo.z}, 350*FRACUNIT)
				if p.mo.percent > 150 then S_StartSound(p.mo, sfx_s3k4b) end
			end

			S_StartSound(p.mo, hit.sound or sfx_pstop)

			p.mo.hitEffect = {hit.effect, 70}
			if s.hitEffectTable[hit.effect] then s.hitEffectTable[hit.effect](p.mo, hit.xfling or 0, hit.yfling or 0) end
			
			// Inflict Status
			if hit.status 
				if hit.status == EFF_BURN or hit.status == EFF_FREEZE
					local statusTime = {
						[EFF_BURN] = 6*TICRATE,
						[EFF_FREEZE] = 3*TICRATE
					}

					p.mo.statusEffect = {hit.status, statusTime[hit.status]} 
				end
			end

			local hitStun = {
				[0] = 1,
				[1] = 5,
				[2] = 10,
				[3] = 15,
				[4] = 20,
			}

			if s.curAttack(p.mo).superarmor or (p.mo.attacktype and p.mo.smashRings and p.mo.smashRings >= 30)
				p.mo.atkstop = h.hitlag or hitStun[hit.priority]
				p.mo.hitcool = h.hitcool or 10

				if h.usermo and h.usermo.valid 
					h.usermo.atkstop = p.mo.flingWait and p.mo.flingWait.time or hitStun[hit.priority]
				end

				return
			end

			p.mo.hitstun = (p.mo.percent/4)+10

			// launch?
			local priorityLaunch = {
				[0] = 300,
				[1] = 180,
				[2] = 120,
				[3] = 100,
				[4] = 80,
			}

			if hit.modify
				if priorityLaunch[hit.priority]
				and p.mo.percent >= (priorityLaunch[hit.priority]-s.invWeight(p.mo.skin))
					p.mo.launch = true
				end
			end

			p.mo.flingWait = {
				time = h.hitlag or hitStun[hit.priority],
				angle = h.angle,
				thrust = 0,
				momz = 0,
				attacker = p.mo,
				hitbox = h,
			}

			if hit.modify
				if hit.xfling and hit.yfling
					p.mo.flingWait.thrust = hit.xfling+(p.mo.percent/6)*(FRACUNIT/7)
					p.mo.flingWait.momz = hit.yfling+(p.mo.percent/5)*(FRACUNIT/3)
				else
					if P_IsObjectOnGround(p.mo)
						p.mo.flingWait.momz = (p.mo.percent/5)*(FRACUNIT/4)
					else
						p.mo.flingWait.momz = ((p.mo.z-h.z)/10)+(p.mo.percent/6)*FRACUNIT/3
					end

					p.mo.flingWait.angle = R_PointToAngle2(h.x, h.y, p.mo.x, p.mo.y)
					p.mo.flingWait.thrust = (10+s.invWeight(p.mo.skin)+(p.mo.percent/6))*(FRACUNIT/4)
				end

				if h.hitMod
					p.mo.flingWait.thrust = ($/100)*h.hitMod
					p.mo.flingWait.momz = ($/100)*h.hitMod
				end
			else
				if hit.xfling and hit.yfling
					p.mo.flingWait.thrust = hit.xfling
					p.mo.flingWait.momz = hit.yfling
				else
					if not P_IsObjectOnGround(p.mo) then p.mo.flingWait.momz = ((p.mo.z-h.z)/12) end
					p.mo.flingWait.angle = R_PointToAngle2(h.x, h.y, p.mo.x, p.mo.y)
					p.mo.flingWait.thrust = (10+s.invWeight(p.mo.skin))*(FRACUNIT/5)
				end
			end

			-- make sure the user is stopped for the same time the hitlag is effective for
			if h.usermo and h.usermo.valid then h.usermo.atkstop = p.mo.flingWait.time end

			-- Stale Moves
			if h.usermo and h.usermo.staleMove and h.usermo.staleMove[2] >= 3
				p.mo.flingWait.thrust = ($/100)*75
				p.mo.flingWait.momz = ($/100)*75
			end
			
			-- Launching
			if p.mo.launch and hit.modify
				p.mo.flingWait.thrust = $*3
				p.mo.flingWait.momz = ($*5)/2

				-- just die like oh my g
				if p.mo.percent >= 220 then p.mo.flingWait.thrust = $*2 end
			end

			-- Meteor Smash sound
			if p.mo.flingWait.momz <= -5*FRACUNIT then S_StartSound(p.mo, sfx_meteor) end

			-- check for last player
			if cv_stocks.value
				local aliveCount = 0
				for player in players.iterate
					if player.lives > 0 and not player.smashSpectate
						aliveCount = $+1
					end
				end

				if aliveCount <= 2 and p.lives <= 1 and p.mo.launch
				and (p.mo.flingWait.thrust >= 35*FRACUNIT or p.mo.flingWait.momz >= 20*FRACUNIT or p.mo.flingWait.momz <= -50*FRACUNIT)
					s.finalHit(p.mo, h.usermo or nil)
				end
			end
		else
			S_StartSound(p.mo, sfx_s3k7b)

			if hit.xfling and hit.yfling
				P_InstaThrust(p.mo, h.angle, hit.xfling/2)
				P_SetObjectMomZ(p.mo, hit.yfling/2)
			else
				if not P_IsObjectOnGround(p.mo) then p.mo.momz =((p.mo.z-(h.z))/9) end
				P_Thrust(p.mo, R_PointToAngle2(h.x, h.y, p.mo.x, p.mo.y), (10+s.invWeight(p.mo.skin))*(FRACUNIT/4))
			end
		end

		p.mo.hitcool = h.hitcool or 10

		// Elec Stun
		if hit.status == EFF_ELECTRICSTUN
			p.mo.flingWait.time = $+TICRATE

			local thok = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_THOK)
			thok.sprite = SPR_ENRG
			thok.scale = FRACUNIT/3
			thok.destscale = 2*FRACUNIT
			thok.tics = 20

			for i = 1, 8
				local electric = P_SpawnMobj(p.mo.x + P_RandomRange(-20, 20)*FRACUNIT, p.mo.y + P_RandomRange(-20, 20)*FRACUNIT, p.mo.z + P_RandomRange(-20, 20)*FRACUNIT, MT_THOK)
				electric.sprite = SPR_WZAP
				electric.frame = A
				electric.tics = 35
				electric.fuse = 35
				electric.scale = P_RandomRange(FRACUNIT, 2*FRACUNIT)
				electric.momx = P_RandomRange(-15, 15)*FRACUNIT
				electric.momy = P_RandomRange(-15, 15)*FRACUNIT
				electric.momz = P_RandomRange(-15, 15)*FRACUNIT
			end
			
			p.mo.wasElec = true
		end
		
		// Shield
		if h.usermo.shield
			if h.usermo.shield.incKnock
				p.mo.flingWait.thrust = ($/100)*h.usermo.shield.incKnock
				p.mo.flingWait.momz = ($/100)*h.usermo.shield.incKnock
			end
		end

		if p.mo.shield
			if p.mo.shield.knockback
				p.mo.flingWait.thrust = ($/100)*p.mo.shield.knockback
				p.mo.flingWait.momz = ($/100)*p.mo.shield.knockback
			end

			p.mo.shield.hp = $-1
			if p.mo.shield.hp <= 0
				p.mo.shield = nil
				S_StartSound(p.mo, sfx_s3k5d)
			end
		end
	end
end

s.ghostHitbox = function(p, h)
	if h and h.valid and h.hitbox and p.mo and p.mo.valid
		if p.mo.hitcool then return false end
		if p.powers[pw_flashing] then return false end

		local hit = h.hitbox
		s.addDamage(p.mo, nil, hit.damage)

		if hit.priority <= 1
//				S_StartSound(p.mo, sfx_s3k49)
		elseif hit.priority == 2
			P_StartQuake(FRACUNIT, 10, {p.mo.x, p.mo.y, p.mo.z}, 200*FRACUNIT)
//				S_StartSound(p.mo, sfx_pstop)
		elseif hit.priority >= 3
			P_StartQuake(3*FRACUNIT, 10, {p.mo.x, p.mo.y, p.mo.z}, 350*FRACUNIT)
//				S_StartSound(p.mo, sfx_pstop)
			if p.mo.percent > 150 then S_StartSound(p.mo, sfx_s3k4b) end
		end

		S_StartSound(p.mo, hit.sound or sfx_pstop)
		p.mo.hitEffect = {hit.effect, 70}

		local hitStun = {
			[0] = 1,
			[1] = 3,
			[2] = 6,
			[3] = 9,
			[4] = 12,
		}

		if s.curAttack(p.mo).superarmor or (p.mo.attacktype and p.mo.smashRings and p.mo.smashRings >= 30)
			p.mo.atkstop = h.hitlag or hitStun[hit.priority]
			p.mo.hitcool = h.hitcool or 10
			return
		end

		p.mo.hitstun = (p.mo.percent/4)+10

		// launch?
		local priorityLaunch = {
			[0] = 300,
			[1] = 180,
			[2] = 120,
			[3] = 100,
			[4] = 80,
		}

		if hit.modify
			if priorityLaunch[hit.priority]
			and p.mo.percent >= (priorityLaunch[hit.priority]-s.invWeight(p.mo.skin))
				p.mo.launch = true
			end
		end

		p.mo.flingWait = {
			time = h.hitlag or hitStun[hit.priority],
			angle = h.angle,
			thrust = 0,
			momz = 0,
			hitbox = h,
		}

		if hit.modify
			if hit.xfling and hit.yfling
				p.mo.flingWait.thrust = hit.xfling+(p.mo.percent/6)*(FRACUNIT/7)
				p.mo.flingWait.momz = hit.yfling+(p.mo.percent/5)*(FRACUNIT/3)
				if p.mo.flingWait.momz <= -5*FRACUNIT then S_StartSound(p.mo, sfx_meteor) end
			else
				if P_IsObjectOnGround(p.mo)
					p.mo.flingWait.momz = (p.mo.percent/5)*(FRACUNIT/4)
				else
					p.mo.flingWait.momz = ((p.mo.z-h.z)/10)+(p.mo.percent/6)*FRACUNIT/3
				end

				p.mo.flingWait.angle = R_PointToAngle2(h.x, h.y, p.mo.x, p.mo.y)
				p.mo.flingWait.thrust = (10+s.invWeight(p.mo.skin)+(p.mo.percent/6))*(FRACUNIT/4)
			end

			if h.hitMod
				p.mo.flingWait.thrust = ($/100)*h.hitMod
				p.mo.flingWait.momz = ($/100)*h.hitMod
			end
		else
			if hit.xfling and hit.yfling
				p.mo.flingWait.thrust = hit.xfling
				p.mo.flingWait.momz = hit.yfling
			else
				if not P_IsObjectOnGround(p.mo) then p.mo.flingWait.momz = ((p.mo.z-h.z)/12) end
				p.mo.flingWait.angle = R_PointToAngle2(h.x, h.y, p.mo.x, p.mo.y)
				p.mo.flingWait.thrust = (10+s.invWeight(p.mo.skin))*(FRACUNIT/5)
			end
		end

		if p.mo.launch
			p.mo.flingWait.thrust = $*3
			p.mo.flingWait.momz = ($*5)/2
			p.mo.flingWait.time = $*2

			-- just die like oh my g
			if p.mo.percent >= 220 then p.mo.flingWait.thrust = $*3 end
		end

		-- check for last player
		if cv_stocks.value
			local aliveCount = 0
			for player in players.iterate
				if player.lives > 0 and not player.smashSpectate
					aliveCount = $+1
				end
			end

			if aliveCount <= 2 and p.lives <= 1 and p.mo.launch and (p.mo.flingWait.thrust >= 35*FRACUNIT or p.mo.flingWait.momz >= 20*FRACUNIT or p.mo.flingWait.momz <= -50*FRACUNIT)
				s.finalHit(p.mo)
			end
		end
	end
end