local s = SRB2Smash

// LOAD GAME
s.inTable = function(val, table)
	if not table then return false end
	if not #table then return false end

	for k,v in pairs(table)
		if v == val then return true end
	end
	
	return false
end

s.resetPlayers = function()
	-- Don't do this in Sudden Death
	if server.suddenDeath then return end

	-- Player values that need resetting when the map changes
	for p in players.iterate
		p.smashPoints = 0
		p.smashStocks = cv_stockcount.value

		p.finalScoreStuff = {
			maxcombo = 0,
			ringtotal = 0,
			totaldmgdealt = 0,
			totaldmgtaken = 0,
			parries = 0,
			attackshit = 0,
			attacksdone = 0
		}
	end

	-- Elect players to be in the next game
	if not cv_plimit.value then 
		local playerMobjs = {}
		for p in players.iterate
--			p.smashSpectate = true
			p.supposedToBeInThisGame = nil
			table.insert(playerMobjs, p.mo)
		end

		server.lastGame = playerMobjs
		return
	end
	
	if not server.lastGame then server.lastGame = {} end

	local playerCount = {}
	for p in players.iterate
		p.smashSpectate = true
		p.supposedToBeInThisGame = nil

		table.insert(playerCount, p)
	end
	
	if #playerCount <= cv_matchlimit.value
		for p in players.iterate
			p.smashSpectate = nil
			p.supposedToBeInThisGame = true
			p.mo.state = S_PLAY_STND
			p.playerstate = PST_REBORN
		end
		
		print("There's only "..cv_matchlimit.value.." players anyway, so this game is between them!")
		return false
	end

	local fighters = {}
	local noResearch = {}
	while #fighters < max(2, cv_matchlimit.value)
		local pnum = P_RandomRange(1, #playerCount)
		if not s.inTable(playerCount[pnum], fighters) and not s.inTable(server.lastGame) and #noResearch <= #fighters
			playerCount[pnum].supposedToBeInThisGame = true
			table.insert(fighters, playerCount[pnum])
		end

		if not s.inTable(playerCount[pnum], noResearch)
			table.insert(noResearch, playerCount[pnum])
		end
	end

	if not #fighters
		for p in players.iterate
			p.supposedToBeInThisGame = true

			p.smashSpectate = nil
			p.mo.state = S_PLAY_STND
			p.playerstate = PST_REBORN
		end
		
		print("Something went wrong, so this game is now a free for all!")
		return false
	end

	for k,v in pairs(fighters)
		v.supposedToBeInThisGame = true

		v.smashSpectate = nil
		v.mo.state = S_PLAY_STND
		v.playerstate = PST_REBORN
	end

	server.lastGame = fighters
	return true
end

// POINTS SCREEN
s.pointStop = function(mo)
	if not mo.saveState
		mo.saveState = {
			state = mo.state,
			sprite = mo.sprite,
			sprite2 = mo.sprite2 or nil,
			frame = mo.frame,
			tics = mo.tics
		}
	end

	mo.momx = 0
	mo.momy = 0
	mo.momz = 0
	mo.state = mo.saveState.state
	mo.sprite = mo.saveState.sprite
	if mo.saveState.sprite2 then mo.sprite2 = mo.saveState.sprite2 end
	mo.frame = mo.saveState.frame
end

s.pointButtons = function(p)	
	p.pointButtons = $ or {
		up = 0,
		down = 0,
		jump = 0,
		spin = 0
	}

	if (p.cmd.buttons & BT_JUMP)
		p.pointButtons.jump = $+1
	else
		p.pointButtons.jump = 0
	end

	if (p.cmd.buttons & BT_SPIN)
		p.pointButtons.spin = $+1
	else
		p.pointButtons.spin = 0
	end

	if (p.cmd.forwardmove < 0)
		p.pointButtons.down = $+1
	else
		p.pointButtons.down = 0
	end

	if (p.cmd.forwardmove > 0)
		p.pointButtons.up = $+1
	else
		p.pointButtons.up = 0
	end
end

s.pointsScreenPlayer = function(p)
	s.pointButtons(p)

	p.screenData = $ or {
		yscore = 1,
		ready = false,
	}
	
	if p.pointButtons.down
		p.screenData.yscore = $+2
	elseif p.pointButtons.up and (p.screenData.yscore > 0)
		p.screenData.yscore = $-2
	end

	if (p.pointButtons.jump == 1) and (p.screenData.ready == false)
		p.screenData.ready = true
		S_StartSound(nil, sfx_kc4a, p)
	elseif (p.pointButtons.spin == 1) and (p.screenData.ready == true)
		p.screenData.ready = false
		S_StartSound(nil, sfx_kc4a, p)
	end
end

s.pointsScreen = function()
	for p in players.iterate
//		s.removeMovement(p)

		p.tutorialShit = nil
		s.pointStop(p.mo)
	end
	
	S_ChangeMusic("VSRSLT")

	if not server.voteScreen
		if not server.pointHud
			server.pointHud = {
				winnerX = -50,
				pointsX = 320,
				timer = 0,
			}
		end
		
		server.pointHud.timer = $+1
		if server.pointHud.timer <= 5
			server.pointHud.winnerX = $+4
		elseif server.pointHud.timer <= 3*TICRATE
			if not (leveltime%2) then server.pointHud.winnerX = $+1 end
		else
			if server.pointHud.winnerX > 20
				server.pointHud.winnerX = $-3
				if server.pointHud.winnerX < 20
					server.pointHud.winnerX = 20
				end
			end
			
			if server.pointHud.pointsX > 170
				server.pointHud.pointsX = $-50
				if server.pointHud.pointsX < 170
					server.pointHud.pointsX = 170
				end
			end
		end

		if server.pointHud.timer >= 10+(3*TICRATE)
			if not server.forceReadyTimer then server.forceReadyTimer = 20*TICRATE end

			server.forceReadyTimer = $-1

			local ready = 0
			local pCount = 0
			for player in players.iterate
				s.pointsScreenPlayer(player)
				pCount = $+1
				
				if server.forceReadyTimer <= 0
					if player.screenData then player.screenData.ready = true end
				end
				
				if player.screenData and player.screenData.ready
					ready = $+1
				end
			end
			
			if ready >= pCount
				server.pointHud = nil
				server.voteScreen = true
				server.forceReadyTimer = nil
			end
		end
	else
		server.winningPlayer = nil
		server.winningTeam = nil
		s.voteThinker()
	end
end