local s = SRB2Smash


s.items = {
	["badniktrophy"] = {
		name = "Badnik Trophy",
		type = "Summoning",
		frame = A,
		hud = {SPR_IPNL, A},
		chance = 2,

		canpickup = function(mo, panel)
			if mo.curItem == "badniktrophy" then return false end
			return true
		end,

		usefunc = function(mo)
			local missile = s.fireMissile(mo, MT_BADNIKTROPHY, 3)
			if missile.valid
				missile.color = mo.color
				missile.colorized = true

				missile.momz = 4*FRACUNIT*P_MobjFlip(mo)
			end
		end
	},

	["pityshield"] = {
		name = "Pity Shield",
		type = "Status",
		frame = B,
		hud = {SPR_IPNL, B},
		chance = 5,

		canpickup = function(mo, panel)
			if mo.shield then return false end
			return true
		end,

		useOnCollect = true,
		collectfunc = function(mo)
			S_StartSound(mo, sfx_s1af)

			mo.shield = {
				def = 2,
				hp = 4,
				knockback = 100,
				time = 30*TICRATE,
				mobj = MT_PITY_ORB,
			}
		end
	},

	["forceshield"] = {
		name = "Force Shield",
		type = "Status",
		frame = C,
		hud = {SPR_IPNL, C},
		chance = 4,

		canpickup = function(mo, panel)
			if mo.shield then return false end
			return true
		end,

		useOnCollect = true,
		collectfunc = function(mo)
			S_StartSound(mo, sfx_forcsg)

			mo.shield = {
				def = 3,
				hp = 7,
				knockback = 50,
				time = 30*TICRATE,
				mobj = MT_FORCE_ORB,
			}
		end
	},

	["attractionshield"] = {
		name = "Attraction Shield",
		type = "Status",
		frame = D,
		hud = {SPR_IPNL, D},
		chance = 3,

		canpickup = function(mo, panel)
			if mo.shield then return false end
			return true
		end,

		useOnCollect = true,
		collectfunc = function(mo)
			S_StartSound(mo, sfx_forcsg)

			mo.shield = {
				def = 2,
				hp = 5,
				knockback = 100,
				ringSteal = true,
				time = 30*TICRATE,
				mobj = MT_ATTRACT_ORB,
			}
		end
	},

	["elementalshield"] = {
		name = "Elemental Shield",
		type = "Status",
		frame = E,
		hud = {SPR_IPNL, E},
		chance = 2,

		canpickup = function(mo, panel)
			if mo.shield then return false end
			return true
		end,

		useOnCollect = true,
		collectfunc = function(mo)
			S_StartSound(mo, sfx_forcsg)

			mo.shield = {
				def = 1,
				hp = 4,
				knockback = 100,
				time = 20*TICRATE,
				mobj = MT_ELEMENTAL_ORB,
			}
		end
	},

	["whirlwindshield"] = {
		name = "Whirlwind Shield",
		type = "Status",
		frame = F,
		hud = {SPR_IPNL, F},
		chance = 2,

		canpickup = function(mo, panel)
			if mo.shield then return false end
			return true
		end,

		useOnCollect = true,
		collectfunc = function(mo)
			S_StartSound(mo, sfx_forcsg)

			mo.shield = {
				def = 1,
				hp = 5,
				incKnock = 150,
				knockback = 125,
				time = 20*TICRATE,
				mobj = MT_WHIRLWIND_ORB,
			}
		end
	},

	["growth"] = {
		name = "Growth",
		type = "Status",
		frame = G,
		hud = {SPR_IPNL, G},
		chance = 2,

		canpickup = function(mo, panel)
			if mo.growth then return false end
			return true
		end,

		usefunc = function(mo)
			mo.growth = 15*TICRATE
			mo.destscale = 2*FRACUNIT
		end
	},

	["spikeball"] = {
		name = "Spike Ball",
		type = "Special",
		chance = 3,
		
		noPanel = MT_SMASH_SPIKEBALL,
		fuse = 30*TICRATE
	}
}

rawset(_G, "MF_ITEMSPAWN", 0x40000000)

s.spawnRandItem = function()
	local itemTypes = {}
	for k,v in pairs(s.items)
		for i = 1, v.chance
			table.insert(itemTypes, k)
		end
	end
	
	local possibleMobj = {}
	for mo in mobjs.iterate()
		if mo and mo.valid and (mo.flags & MF_ITEMSPAWN)
			table.insert(possibleMobj, mo)
		end
	end

	local mobj = possibleMobj[P_RandomRange(1, #possibleMobj)]
	if mobj and mobj.valid
		local x, y, z = mobj.x, mobj.y, mobj.ceilingz-100*FRACUNIT

		local itemval = itemTypes[P_RandomRange(1, #itemTypes)]
		if s.items[itemval] and s.items[itemval].noPanel
			local item = P_SpawnMobj(x + P_RandomRange(-400, 400)*FRACUNIT, y, z, s.items[itemval].noPanel)
			item.fuse = s.items[itemval].fuse or 500
		else
			local panel = P_SpawnMobj(x + P_RandomRange(-400, 400)*FRACUNIT, y, z, MT_ITEMPANEL)
			panel.scale = 1
			panel.destscale = FRACUNIT
			panel.itemType = itemval
			panel.fuse = 500
		end
	end
end

s.itemThink = function()
	if not cv_items.value then return end

	if (leveltime > 4*TICRATE)
		local incrementValues = {350, 735, 1050}
		local inc = incrementValues[cv_itemspawnrate.value]

		if not (leveltime%inc)
			s.spawnRandItem()
		end
	end
end

-- mobjspawn shit
addHook("MobjSpawn", function(mo)
	if not s.isSmash() then return end

	if (mo.flags & MF_MONITOR)
		mo.flags = MF_SCENERY|MF_ITEMSPAWN
		mo.flags2 = MF2_DONTDRAW
		return
	end
end)