local s = SRB2Smash

s.tutorials = {
	[1] = {
		title = "BASICS",
		txt = "SRB2Smash consists of a set of gamemodes in which you try to knock your foes off of the various stages you can play on.",
	},

	[2] = {
		title = "CONTROLS",
		txt = "SRB2Smash uses three buttons, Jump, Spin & Fire. In SRB2Smash, you cannot use your character's abilities; instead, you get a double jump just like real Smash Bros!"
	},

	[3] = {
		title = "CONTROLS",
		txt = "To perform a Jab, tap Spin on the ground when not moving. To do a Tilt, hold any direction while pressing Spin instead."
	},

	[4] = {
		title = "CONTROLS",
		txt = "Performing Aerials is like Grounded Attacks, except in the air. Often, Down Airs can send your target flying directly downwards! This is perfect when offstage."
	},

	[5] = {
		title = "CONTROLS",
		txt = "Special Attacks, or Specials are unique to each character, hence the name Specials. You'll have to find out what each does yourself! To perform them, press fire, and any direction."
	},

	[6] = {
		title = "CONTROLS",
		txt = "Blast Attacks are strong attacks that often really high knockback! You can use a forward blast, up blast or down blast. To use them, hold spin while on the ground, and then release when you begin flashing while holding up, down, left or right."
	},

	[7] = {
		title = "CONTROLS",
		txt = "Parries can stun any attacker that hits you - or repel any projectile - if you time it well enough. Press fire-normal on ground to use it. You can also perform a dodge roll to negate damage if timed well. Press left or right while holding down on the ground to do this. You can ALSO ALSO use an Air Dodge to confuse foes if timed well. Press fire-normal mid air to use it."
	},

	[8] = {
		title = "NETGAMES",
		txt = "You are currently spectating a game of SRB2Smash... if there are players in it anyway. If you're ready to join it, exit out of this tutorial with Spin, and enter the game with TossFlag, if it hasn't been too long."
	}
}

s.wrapTxt = function(string, length, v)
	local lastLine = 1
	local newLines = {}
	local lastSpace = 0
	for i = 1, string:len()
		local curLine = string:sub(lastLine, i)
		local curLetter = string:sub(i, i)

		if curLetter == " " then lastSpace = i end

		if (v.stringWidth(curLine) > length) and (curLetter == " ")
			table.insert(newLines, i)
			lastLine = i
		end
	end
	
	if not #newLines then return end
	
	local newString = ""
	for k = 1, string:len()
		local str = string:sub(k, k)
		for i = 1, #newLines
			if str == " " and k == newLines[i] then str = "\n" end
		end

		newString = $..str
	end

	return newString
end

s.openTutorial = function(p)
	p.tutorialShit = {
		curTut = 1,
		openTics = 70,
		tutorials = {}
	}
	
	for i in pairs(s.tutorials)
		p.tutorialShit.tutorials[i] = s.tutorials[i]
	end

	local attacks = {
		"jab1",
		"dash",
		"sidetilt",
		"uptilt",
		"downtilt",
		"neutralair",
		"upair",
		"downair",
		"sideair",
		"neutralspecial",
		"sidespecial",
		"upspecial",
		"downspecial",
		"blast",
		"sideblast",
		"upblast",
		"finalsmash"
	}

	for i = 1, #attacks
		if not s.chars[p.mo.skin][attacks[i]] then continue end

		local v = s.chars[p.mo.skin][attacks[i]]
		if v.desc then
			local titleShit = ""
			if not v.desc.name
				titleShit = p.mo.skin:upper()
			else
				titleShit = v.desc.name
			end

			local descShit = ""
			if not v.desc.txt
				descShit = "Huh. There's no description listed for this move... that's... very interesting."
			else
				descShit = v.desc.txt
			end

			p.tutorialShit.tutorials[#p.tutorialShit.tutorials+1] = {
				title = titleShit,
				txt = descShit
			}
		end
	end
end

s.closeTutorial = function(p)
	p.tutorialShit.closing = true
end

s.tutorialThinker = function(p)
	if not p.tutorialShit then return false end
	p.smashSpectate = true
	
	p.mo.momx = 0
	p.mo.momy = 0
	p.mo.momz = 0

	if p.tutorialShit.openTics <= 0 and p.mo.button
		if (p.mo.button.left == 1 or (p.mo.button.left >= 15 and not (leveltime%3))) 
		and p.tutorialShit.curTut > 1
			p.tutorialShit.curTut = $-1
			S_StartSound(nil, sfx_s3k5b, p)
		end

		if (p.mo.button.right == 1 or (p.mo.button.right >= 15 and not (leveltime%3))) 
		and p.tutorialShit.curTut < #p.tutorialShit.tutorials
			p.tutorialShit.curTut = $+1
			S_StartSound(nil, sfx_s3k5b, p)
		end
		
		if p.mo.button.spin == 1
			S_StartSound(nil, sfx_s3k5b, p)
			s.closeTutorial(p)
		end
	end

	if p.tutorialShit.closing
		p.tutorialShit.openTics = $+2
		if p.tutorialShit.openTics >= 70
			p.tutorialShit = nil
			return true
		end
	else
		if p.tutorialShit.openTics > 0 then p.tutorialShit.openTics = $-2 end
	end
end

s.tutorialHud = function(v, p)
	if p.tutorialShit
		local tut = p.tutorialShit
		
		local x, y = -64+(leveltime%64), 165+tut.openTics
		v.drawScaled(x*FRACUNIT, y*FRACUNIT, 2*FRACUNIT, v.cachePatch("TUTORIAL2"), V_SNAPTOBOTTOM)

		x, y = -(leveltime%64), -tut.openTics
		v.drawScaled(x*FRACUNIT, y*FRACUNIT, 2*FRACUNIT, v.cachePatch("TUTORIAL1"), V_SNAPTOTOP)

		if tut.tutorials[tut.curTut]
			local title = tut.tutorials[tut.curTut].title
			local text = s.wrapTxt(tut.tutorials[tut.curTut].txt, 230, v)
			v.drawNameTag(130, 40, title, V_SNAPTOTOP, SKINCOLOR_RED, SKINCOLOR_BLUE)
			v.drawString(5, 119, text, V_SNAPTOBOTTOM|V_ALLOWLOWERCASE)
		end
	end
end