addHook("TouchSpecial", SRB2Smash.getRing, MT_RING)
addHook("TouchSpecial", SRB2Smash.getRing, MT_SMASH_RING)

// Hitbox Collision

/*
	BASICALLY
	We're gonna count the hitboxes the player collides with, using TouchSpecial and PreThinkFrame, and fit them in a table.
	Then, we're gonna order the table with highest priority AFTER the rest of the thinkers with a PostThinkFrame, and then damage the players that are hit by the hitboxes.
	That way, we can give priority to higher priority hitboxes, and therefore, implement things like sweet-spots.
	It's very cool
*/

// Set up the Variables
addHook("PreThinkFrame", function()
	if not SRB2Smash.isSmash() then return end

	for player in players.iterate
		if player.mo and player.mo.valid
			player.mo.inHitBox = {}
		end
	end
end)

addHook("PostThinkFrame", function()
	// Order the Hitboxes First
	for p in players.iterate
		if p.smashSpectate then continue end
		if p.quittime then continue end
		if p.mo.killed then continue end
		if not p.mo.inHitBox then continue end
		if not #p.mo.inHitBox then continue end

		table.sort(p.mo.inHitBox, function(a, b) 
			if a.valid and b.valid then return (a.priority or 1) > (b.priority or 1) end
			return false
		end)
	end

	// Strike players with the hitbox that is deemed as the highest priority one
	for p in players.iterate
		if p.mo.attacktype ~= "finalsmash" and not p.mo.dodge
			for i = 1, #p.mo.inHitBox
				if p.mo.inHitBox[i] and p.mo.inHitBox[i].valid
//					if p.mo.inHitbox[i].user
						SRB2Smash.hitboxCollide(p.mo.inHitBox[i], p)
//					else
//						SRB2Smash.mobjControlledHitboxCollide(p.mo.inHitBox[i], p)
//					end

					break
				end
			end
		end
	end
	
	// Clear hitbox data
	for p in players.iterate
		if p.mo and p.mo.valid then p.mo.inHitBox = {} end
	end
end)

addHook("TouchSpecial", function(hit, mo)
	if not SRB2Smash.isSmash() then return true end

	if hit.valid and mo.valid and mo.player
		if mo.player.smashSpectate then return true end
		if mo.player.quittime then return true end
		if mo.killed then return true end

		if mo.attacktype ~= "finalsmash"
			if not mo.inHitBox then mo.inHitBox = {} end
			table.insert(mo.inHitBox, hit)
		end
	end

	return true
end, MT_HITBOXSMASH)

// damaage
addHook("MobjMoveCollide", function(thing, tmthing)
	if not SRB2Smash.isSmash() then return end
	
	if thing and tmthing and tmthing.type == MT_HITBOXSMASH
	and thing.valid and tmthing.valid and tmthing.user and tmthing.user.valid
		if thing.z >= tmthing.z + tmthing.height
		or thing.z + thing.height < tmthing.z
			return false
		end

		if thing.type == MT_SMASHBALL 
		or thing.type == MT_SMASHBALL_FAKE 
		or thing.type == MT_SMASHBALL_GROUNDED
			tmthing.usermo.lastHit = 0
			SRB2Smash.smashBallDamage(thing, tmthing.usermo, tmthing)
			return
		end
		
		if thing.type == MT_SMASH_SPIKEBALL and not thing.hitcool and not thing.repelMovement
			S_StartSound(thing, sfx_pstop)
			thing.user = tmthing.usermo
			thing.target = tmthing.usermo			
			thing.color = tmthing.usermo.player and tmthing.usermo.player.skincolor or tmthing.usermo.color
			thing.colorized = true

			tmthing.usermo.atkstop = 5
			tmthing.usermo.lastHit = 0

			thing.hitcool = 5
			thing.repelMovement = 5
			thing.resumeMove = {
				x = tmthing.xfling,
				y = tmthing.momy,
				z = tmthing.yfling,
				state = thing.state
			}
		end

		if (thing.flags & MF_MISSILE)
			SRB2Smash.missileRepel(tmthing, thing)
			tmthing.usermo.lastHit = 0
			return
		end
	end
end)

// deth
addHook("PlayerThink", function(p)
	if not SRB2Smash.isSmash() then return end
	if p.mo.killed then return end
	if p.mo.killedBlast then return end
	if p.mo.starKO then return end
	if p.mo.screenKO then return end

	for mobj in mobjs.iterate()
		if mobj and mobj.valid
			if mobj.type == MT_UPBLAST
				if p.mo.z >= mobj.z and p.mo.attacker and p.mo.attacker.valid
					if P_RandomFixed() >= FRACUNIT/2
						SRB2Smash.checkKO(p.mo, p.mo.attacker)
					else
						SRB2Smash.killMobj(p.mo, p.mo.attacker, mobj)
					end
				end
			elseif mobj.type == MT_SIDEBLAST
				if p.mo.x <= mobj.x+(mobj.radius/2) and p.mo.x >=mobj.x-(mobj.radius/2)
					if p.mo.attacker and p.mo.attacker.valid
						SRB2Smash.killMobj(p.mo, p.mo.attacker, mobj)
					else
						SRB2Smash.killMobj(p.mo, nil, mobj)
					end
				end
			end
		end
	end
end)