addHook("PlayerThink", function(p)
	if not SRB2Smash.isSmash() then return end
	
	/*
		Things that run reguardless of playerstate & quittime
		If the player is dead, these things run anyway.
	*/
	
	if p.mo and p.mo.valid then SRB2Smash.handleDieCooldown(p.mo) end

	-- Update HUD Values.
	SRB2Smash.updateHud(p)
	SRB2Smash.blastZoneCheck(p)
	
	-- Tutorial Thinker
	SRB2Smash.tutorialThinker(p)

	-- Spectator thinker
	if not p.supposedToBeInThisGame
		SRB2Smash.spectatorToggle(p)
		if p.smashSpectate or p.mo.tempSpectate
			SRB2Smash.spectatorThinker(p)
			return
		end
	end

	-- death
	SRB2Smash.killedBlast(p)
	if p.mo and p.mo.valid then SRB2Smash.blastZoneEff(p.mo) end
	
	-- Camera Thinker
	SRB2Smash.cameraThinker(p)

	-- after death funcs
	if p.mo.killed then return end
	if p.quittime then return end
	if p.playerstate == PST_DEAD then return end

	-- Inputs
	if p and p.valid
		SRB2Smash.inputRegister(p)
		SRB2Smash.updateValues(p)

		if (leveltime > 4*TICRATE) then SRB2Smash.fireInputs(p) end
	end

	-- Attacks and Abilities
	if p.mo and p.mo.valid
		SRB2Smash.spinInputs(p)
		SRB2Smash.attackThinker(p)
		if p.charability == CA_FLY then SRB2Smash.flyFuck(p) end
	end		
end)

addHook("MobjThinker", function(mo)
	if mo.player then return end
	
	SRB2Smash.initBot(mo)
	SRB2Smash.mobjThinker(mo)
end, MT_PLAYER)

addHook("JumpSpecial", function(p)
	if p.mo.shielding then return true end
	if p.mo.parry then return true end
	if p.mo.doBlast then return true end
	if p.mo.killed then return true end
end)

addHook("AbilitySpecial", function(p)
	if not SRB2Smash.isSmash() then return false end

	if p.mo.killed then return true end
	if (p.smashSpectate or p.mo.tempSpectate) then return true end

	if p.mo and p.mo.valid
		if p.mo.attacktype == "finalsmash" then return true end
		SRB2Smash.doubleJump(p)

		return true
	end
end)

addHook("SpinSpecial", function(p)
	if not SRB2Smash.isSmash() then return false end
	if (p.smashSpectate or p.mo.tempSpectate) then return true end
	if p.mo.killed then return true end

	if p.mo and p.mo.valid
		return SRB2Smash.noSpin(p)
	end
end)

addHook("PlayerHeight", function(p)
	if p.mo and p.mo.valid and p.mo.shielding
		return 20*FRACUNIT
	end
end)

addHook("MobjCollide", SRB2Smash.resetSpring, MT_PLAYER)
addHook("MobjMoveCollide", SRB2Smash.resetSpring, MT_PLAYER)

addHook("ShouldDamage", function(mo, inf, src, dmg, dmgType)
	if not SRB2Smash.isSmash() then return end
	if mo.player and (mo.player.smashSpectate or mo.tempSpectate) then return false end
	
	if mo and mo.valid and mo.player and mo.player.valid
		if mo.player.quittime then return false end
		if mo.killed then return false end

		if inf and src and inf.valid and src.valid
			if mo.attacktype == "finalsmash" then return false end
			SRB2Smash.damageFromObj(mo.player, inf, src, true, true)
		elseif inf and inf.valid
			if mo.attacktype == "finalsmash" then return false end
			SRB2Smash.damageFromObj(mo.player, inf, nil, true)
		elseif not (dmgType & DMG_DEATHMASK)
			if mo.attacktype == "finalsmash" then return false end
			if not mo.flingWait and not mo.hitcool
				SRB2Smash.damageFromSector(mo, dmgType)
			end
		elseif (dmgType & DMG_DEATHMASK)
			if mo.killed then return false end
			SRB2Smash.killMobj(mo, mo.attacker, mo)
		end

		return false
	end
end, MT_PLAYER)

// Auto Spectate upon join.
addHook("PlayerJoin", function(p)
	if cv_autospec.value
		if server and server.valid
			server.setSpectate = $ or {}
		end
		
		server.setSpectate[p] = true
	end
end)

addHook("PlayerSpawn", function(p)
	p.killedBlast = nil
	if p.mo and p.mo.valid
		p.mo.dieCool = 10
	end

	// Initiate Player Values
	SRB2Smash.initPlayer(p)

	// Auto Spectate
	if cv_autospec.value
		if server.setSpectate[#p]
			p.smashSpectate = true
			server.setSpectate[#p] = nil
		end
	end
end)